require 'spec_helper'

describe UrlScrubber do
  describe '.scrub' do
    it 'should downcase all domain stuff' do
      UrlScrubber.scrub("http://ExAmple.COM").should eq('http://example.com')
    end

    it 'should remove whitespace and stray characters around an url' do
      UrlScrubber.scrub("d http://example.com\t").should eq("http://example.com")
    end

    it 'should turn an https url into http' do
      UrlScrubber.scrub('https://example.com').should eq('http://example.com')
    end

    it 'should remove trailing slashes' do
      UrlScrubber.scrub('http://example.com/').should eq('http://example.com')
    end

    it 'should remove #!/' do
      UrlScrubber.scrub('http://example.com/#!/page').should eq('http://example.com/page')
    end

    it 'should drop #anchors' do
      UrlScrubber.scrub('http://example.com/page#query:example+pages').should eq('http://example.com/page')
    end

    it 'should drop www' do
      UrlScrubber.scrub('http://www.example.com/page').should eq('http://example.com/page')
    end


    describe 'with youtube urls' do
      it 'should NOT drop /user/ if it exists' do
        UrlScrubber.scrub('http://youtube.com/user/absolutely').should eq('http://youtube.com/user/absolutely')
      end

      it 'should handle /profile?user= urls' do
        UrlScrubber.scrub('http://youtube.com/profile?user=absolutely').should eq('http://youtube.com/absolutely')
      end

      it 'should not kick in for a non-youtube url' do
        UrlScrubber.scrub('http://example.com/profile?user=absolutely').should_not eq('http://example.com/absolutely')
      end
    end


    describe 'with twitter urls' do
      it 'should drop @ from in front of username' do
        UrlScrubber.scrub('http://twitter.com/@absolutely').should eq('http://twitter.com/absolutely')
      end

      it 'should not kick in for a non-twitter url' do
        UrlScrubber.scrub('http://example.com/@absolutely').should_not eq('http://example.com/absolutely')
      end

      it 'should transform /search/realtime/%40username into that username' do
        UrlScrubber.scrub('http://twitter.com/search/realtime/%40absolutely').should eq('http://twitter.com/absolutely')
      end

      it 'should handle similar search urls' do
        UrlScrubber.scrub('http://twitter.com/search/realtime/@absolutely').should eq('http://twitter.com/absolutely')
        UrlScrubber.scrub('http://twitter.com/search/%40absolutely').should eq('http://twitter.com/absolutely')
        UrlScrubber.scrub('http://twitter.com/search/@absolutely').should eq('http://twitter.com/absolutely')
        UrlScrubber.scrub('http://twitter.com/search/realtime?q=%40absolutely').should eq('http://twitter.com/absolutely')
      end

      it "should transform user statuses into that user's profile" do
        UrlScrubber.scrub('http://twitter.com/absolutely/statuses/135243243261312').should eq('http://twitter.com/absolutely')
      end

      it "should drop the query part of the url" do
        UrlScrubber.scrub('http://twitter.com/novartisuk?lang=en').should eq('http://twitter.com/novartisuk')
      end
    end


    describe 'with facebook urls' do
      it 'should drop /home.php?#!/ from the beginning of the path' do
        UrlScrubber.scrub('http://facebook.com/home.php?#!/person.name').should eq('http://facebook.com/person.name')
      end

      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.facebook.com/person.name').should eq('http://facebook.com/person.name')
      end

      it 'should drop an url query' do
        UrlScrubber.scrub('http://facebook.com/person.name?ref=pb').should eq('http://facebook.com/person.name')
      end

      # Vanity URL

      it 'should not change a good vanity url' do
        UrlScrubber.scrub('http://facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should only drop the trailing / on a good vanity url' do
        UrlScrubber.scrub('http://facebook.com/TonyMollHomeLoans/').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should only drop the trailing /about on a good vanity url' do
        UrlScrubber.scrub('http://facebook.com/TonyMollHomeLoans/about').should eq('http://facebook.com/TonyMollHomeLoans')
      end
      
      it 'should only drop the trailing /timeline on a good vanity url' do
        UrlScrubber.scrub('http://facebook.com/TonyMollHomeLoans/timeline').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should only drop the trailing ?_rdr=p on a good vanity url' do
        UrlScrubber.scrub('http://facebook.com/TonyMollHomeLoans?_rdr=p').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      # Vanity URL with period in username

      it 'should not change a good vanity url with a period in username' do
        UrlScrubber.scrub('http://facebook.com/jim.reischling').should eq('http://facebook.com/jim.reischling')
      end

      it 'should only drop the trailing / on a good vanity url with a period in username' do
        UrlScrubber.scrub('http://facebook.com/jim.reischling/').should eq('http://facebook.com/jim.reischling')
      end

      it 'should only drop the trailing /about on a good vanity url with a period in username' do
        UrlScrubber.scrub('http://facebook.com/jim.reischling/about').should eq('http://facebook.com/jim.reischling')
      end
      
      it 'should only drop the trailing /timeline on a good vanity url with a period in username' do
        UrlScrubber.scrub('http://facebook.com/jim.reischling/timeline').should eq('http://facebook.com/jim.reischling')
      end

      it 'should only drop the trailing ?_rdr on a good vanity url with a period in username' do
        UrlScrubber.scrub('http://facebook.com/jim.reischling?_rdr').should eq('http://facebook.com/jim.reischling')
      end

      # Vanity URL with two periods in username

      it 'should not change a good vanity url two periods in username' do
        UrlScrubber.scrub('http://facebook.com/sam.thwaite.5/').should eq('http://facebook.com/sam.thwaite.5')
      end

      # V1 Non-Vanity URLs

      it 'should not change a good V1 non-vanity url without UID' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning')
      end

      it 'should not change a good V1 non-vanity url without UID with trialing slash' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning')
      end

      it 'should not change a good V1 non-vanity url with UID' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521')
      end

      it 'should only drop the trailing /about on a good V1 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521/about').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521')
      end
      
      it 'should only drop the trailing /info on a good V1 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521/info').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521')
      end
      
      it 'should only drop the trailing /timeline on a good V1 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521/timeline').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521')
      end

      it 'should only drop the trailing ?_rdr on a good V1 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521?_rdr').should eq('http://facebook.com/pages/Chris-Stopp-NMLS-257248-Alpine-Mortgage-Planning/152622158131521')
      end

      # V2 Non-Vanity URLs

      it 'should not change a good V2 non-vanity url without UID' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end

      it 'should not change a good V2 non-vanity url without UID with trialing slash' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414/').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end

      it 'should only drop the trailing /about on a good V2 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414/about').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end
      
      it 'should only drop the trailing /info on a good V2 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414/info').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end
      
      it 'should only drop the trailing /timeline on a good V2 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414/timeline').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end

      it 'should only drop the trailing ?_rdr on a good V2 non-vanity url' do
        UrlScrubber.scrub('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414?_rdr').should eq('http://facebook.com/Pinnacle-Capital-Mortgage-Lynette-Lea-Jorden-Reverse-Mortgage-Specialist-702124366560414')
      end

      it 'should only drop the trailing app_267091300008193 on a good Vanity url' do
        UrlScrubber.scrub('http://facebook.com/theloansisters/app_267091300008193').should eq('http://facebook.com/theloansisters')
      end

      it 'should handle profile URL with shorter UID' do
        UrlScrubber.scrub('http://www.facebook.com/profile.php?id=1165522415').should eq('http://facebook.com/profile.php?id=1165522415')
      end

      it 'should handle profile URL with longer UID' do
        UrlScrubber.scrub('http://www.facebook.com/profile.php?id=100004113611106').should eq('http://facebook.com/profile.php?id=100004113611106')
      end

      it 'should handle profile URL with longer UID and https' do
        UrlScrubber.scrub('https://www.facebook.com/profile.php?id=100000735668376').should eq('http://facebook.com/profile.php?id=100000735668376')
      end

      it 'should handle profile URL with fref' do
        UrlScrubber.scrub('https://www.facebook.com/profile.php?id=100007058896205&fref=ts').should eq('http://facebook.com/profile.php?id=100007058896205')
      end

      it 'should handle profile URL with &_rdr' do
        UrlScrubber.scrub('https://www.facebook.com/profile.php?id=100008623904282&_rdr').should eq('http://facebook.com/profile.php?id=100008623904282')
      end

      it 'should handle profile URL with _rdr=p&' do
        UrlScrubber.scrub('https://www.facebook.com/profile.php?_rdr=p&id=100009574328879').should eq('http://facebook.com/profile.php?id=100009574328879')
      end
      
      it 'should handle uppercase HTTP' do
        UrlScrubber.scrub('HTTP://facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle uppercase HTTPS' do
        UrlScrubber.scrub('HTTPS://facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle ww.' do
        UrlScrubber.scrub('http://ww.facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle ww1.' do
        UrlScrubber.scrub('http://ww1.facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle www1.' do
        UrlScrubber.scrub('http://ww1.facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle wwww.' do
        UrlScrubber.scrub('http://wwww.facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should handle mobile urls (m.)' do
        UrlScrubber.scrub('http://m.facebook.com/TonyMollHomeLoans').should eq('http://facebook.com/TonyMollHomeLoans')
      end

      it 'should not kick in for a non-facebook url' do
        UrlScrubber.scrub('http://example.com/home.php?#!/person.name').should_not eq('http://example.com/person.name')
      end
    end


    # LinkedIn

    describe 'with linkedin urls' do
      it 'should change /companies/ to /company/' do
        UrlScrubber.scrub('http://linkedin.com/companies/1337').should eq('http://linkedin.com/company/1337')
      end

      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.linkedin.com/company/1337').should eq('http://linkedin.com/company/1337')
      end

      it 'should drop query parameters from the end of a company url' do
        UrlScrubber.scrub('http://linkedin.com/company/1337?trk=tyah').should eq('http://linkedin.com/company/1337')
      end

      it 'should not drop query parameters a profile view url' do
        UrlScrubber.scrub('http://www.linkedin.com/profile/view?id=12341324').should eq('http://linkedin.com/profile/view?id=12341324')
      end

      it 'should not kick in for a non-linkedin url' do
        UrlScrubber.scrub('http://example.com/companies/1337').should_not eq('http://example.com/company/1337')
      end
    end

    describe 'with google plus urls' do
      it 'should drop anything after the first significant path component' do
        UrlScrubber.scrub('http://plus.google.com/+SomeName/posts').should eq('http://plus.google.com/+SomeName')
      end

      it 'should drop u/0/b/ from the beginning of the path' do
        UrlScrubber.scrub(
          'http://plus.google.com/u/0/b/111111111111111111111'
        ).should eq(
          'http://plus.google.com/111111111111111111111'
        )
      end

      it 'should drop u/0/ from the beginning of the path' do
        UrlScrubber.scrub(
          'https://plus.google.com/u/0/5432123454135/posts'
        ).should eq(
          'http://plus.google.com/5432123454135'
        )
      end

      it 'should not kick in for a non-google-plus url' do
        UrlScrubber.scrub('http://example.com/+SomeName/posts').should_not eq('http://example.com/+SomeName')
      end
    end

    describe 'with slideshare urls' do
      skip "is not implemented yet" do
        # pending
      end
    end

    describe 'with flickr urls' do
      it 'should drop /photos/ from profile names' do
        UrlScrubber.scrub('http://flickr.com/photos/username/').should eq('http://flickr.com/username')
      end

      it 'should drop /people/ from profile names' do
        UrlScrubber.scrub('http://flickr.com/people/username/').should eq('http://flickr.com/username')
      end

      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.flickr.com/username').should eq('http://flickr.com/username')
      end

      it 'should drop path components after the username' do
        UrlScrubber.scrub('http://flickr.com/username/favorites').should eq('http://flickr.com/username')
      end

      it 'should not kick in for a non-flickr url' do
        UrlScrubber.scrub('http://example.com/photos/username/').should_not eq('http://example.com/username')
      end
    end

    describe 'with pinterest urls' do
      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.pinterest.com/username').should eq('http://pinterest.com/username')
      end
    end

    describe 'with yelp urls' do
      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.yelp.com/biz/very-important-business').should eq('http://yelp.com/biz/very-important-business')
      end
    end

    describe 'with vimeo urls' do
      skip "is not implemented yet" do
        # pending
      end
    end
    
    describe 'with instagram urls' do
      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.instagram.com/username').should eq('http://instagram.com/username')
      end
    end

    describe 'with tumblr urls' do
      it 'should drop www. from the beginning' do
        UrlScrubber.scrub('http://www.cisco.tumblr.com').should eq('http://cisco.tumblr.com')
      end
    end

  end

  describe '.service_of' do
    it 'returns :youtube for YouTube urls' do
      UrlScrubber.service_of('http://youtube.com/absolutely').should eq :youtube
    end

    it 'returns :twitter for Twitter urls' do
      UrlScrubber.service_of('http://twitter.com/absolutely').should eq :twitter
    end

    it 'returns :facebook for Facebook urls' do
      UrlScrubber.service_of('http://facebook.com/person.name').should eq :facebook
    end

    it 'does not return :facebook for a non-facebook url with facebook in the name' do
      UrlScrubber.service_of('http://foofacebook.com/').should_not eq :facebook
      UrlScrubber.service_of('http://facebookfoo.com/').should_not eq :facebook
    end

    it 'returns :linkedin for LinkedIn urls' do
      UrlScrubber.service_of('http://linkedin.com/company/1337').should eq :linkedin
    end

    it 'returns :google for Google+ urls' do
      UrlScrubber.service_of('http://plus.google.com/+SomeName').should eq :google
    end
    
    it 'returns :slideshare for SlideShare urls' do
      UrlScrubber.service_of('http://slideshare.net/absolutely').should eq :slideshare
    end
    
    it 'returns :flickr for Flickr urls' do
      UrlScrubber.service_of('http://flickr.com/username').should eq :flickr
    end
    
    it 'returns :pinterest for Pinterest urls' do
      UrlScrubber.service_of('http://pinterest.com/username').should eq :pinterest
    end
    
    it 'returns :yelp for Yelp urls' do
      UrlScrubber.service_of('http://yelp.com/very-important-business').should eq :yelp
    end

    it 'returns :vimeo for Vimeo urls' do
      UrlScrubber.service_of('http://vimeo.com/absolutely').should eq :vimeo
    end
    
    it 'returns :instagram for Instagram urls' do
      UrlScrubber.service_of('http://instagram.com/absolutely').should eq :instagram
    end
    
    it 'returns :tumblr for Tumblr urls' do
      UrlScrubber.service_of('http://cisco.tumblr.com').should eq :tumblr
    end
    
    it 'returns :other for other urls' do
      UrlScrubber.service_of('http://example.com/page').should eq :other
    end
  end

  describe '.ideal_form?' do
    it 'scrubs the url beforehand' do
      UrlScrubber.should_receive(:scrub).with('http://example.com/some-page/').and_return('http://example.com/some-page')
      UrlScrubber.ideal_form?('http://example.com/some-page/')
    end

    describe 'for youtube' do
      it 'returns true for apparent channel urls' do
        UrlScrubber.ideal_form?('http://youtube.com/absolutely').should be_truthy
      end

      it 'returns false for videos' do
        UrlScrubber.ideal_form?('http://youtube.com/watch?v=vRGMAW1wzQ8').should be_falsey
      end
    end

    describe 'for twitter' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://twitter.com/absolutely').should be_truthy
      end

      it 'returns false for other pages' do
        UrlScrubber.ideal_form?('http://twitter.com/').should be_falsey
      end
    end

    describe 'for facebook' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://facebook.com/person.name').should be_truthy
        UrlScrubber.ideal_form?('http://facebook.com/profile.php?id=543123521').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://facebook.com/').should be_falsey
      end
    end

    describe 'for linkedin' do
      it 'returns true for apparent company urls' do
        UrlScrubber.ideal_form?('http://linkedin.com/company/1337').should be_truthy
        UrlScrubber.ideal_form?('http://www.linkedin.com/profile/view?id=12341324').should be_truthy
        UrlScrubber.ideal_form?('http://linkedin.com/company/brand-name').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://linkedin.com/jobs/c-Cisco').should be_falsey
      end
    end

    describe 'for google plus' do
      it 'returns true for apparent person urls' do
        UrlScrubber.ideal_form?('http://plus.google.com/+SomeName').should be_truthy
        UrlScrubber.ideal_form?('http://plus.google.com/u/0/b/111111111111111111111').should be_truthy
        UrlScrubber.ideal_form?('https://plus.google.com/u/0/5432123454135/posts').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://plus.google.com/').should be_falsey
      end
    end

    describe 'for slideshare' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://slideshare.net/absolutely').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://slideshare.net/absolutely/how-to-create-great-slides-for-presentations').should be_falsey
      end
    end

    describe 'for flickr' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://flickr.com/username').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://flickr.com/').should be_falsey
      end
    end

    describe 'for pinterest' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://pinterest.com/username').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://pinterest.com/pin/532412513451524').should be_falsey
      end
    end

    describe 'for yelp' do
      it 'returns true for apparent business urls' do
        UrlScrubber.ideal_form?('http://yelp.com/very-important-business').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://yelp.com/user_details?userid=Aheunaobuh-huoanuhbAU').should be_falsey
      end
    end

    describe 'for vimeo' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://vimeo.com/absolutely').should be_truthy
      end

      it 'returns false for video urls' do
        UrlScrubber.ideal_form?('http://vimeo.com/45453874578').should be_falsey
      end
    end

    describe 'for instagram' do
      it 'returns true for apparent user urls' do
        UrlScrubber.ideal_form?('http://instagram.com/username').should be_truthy
      end

      it 'returns false for other urls' do
        UrlScrubber.ideal_form?('http://instagram.com/532513451524').should be_falsey
      end
    end

    describe 'for tumblr' do
      it 'returns true for apparent business urls' do
        UrlScrubber.ideal_form?('http://cisco.tumblr.com').should be_truthy
      end

      it 'returns false for user urls' do
        UrlScrubber.ideal_form?('http://tumblr.com/joe').should be_falsey
      end
    end

    describe 'for other sites' do
      it 'returns true for any other site, really' do
        UrlScrubber.ideal_form?('http://example.com/absolutely/anything').should be_truthy
      end
    end
  end
end